#include "ttl_p.h"


TtIniSection::TtIniSection( TtIniFile *ini_file, const char *name ) {
  this->ini_file = ini_file;
  this->name = NULL;
  this->set_name( name );
}

TtIniSection::~TtIniSection() {
  if ( this->name != NULL ) { delete [] this->name; }
}

TtIniFile *TtIniSection::get_ini_file() {
  return this->ini_file;
}

const char *TtIniSection::get_name() {
  return name;
}

void TtIniSection::set_name( const char *name ) {
  if ( this->name != NULL ) { delete [] this->name; }
  this->name = new char[strlen( name ) + 1];
  strcpy( this->name, name );
}

int TtIniSection::get_int( const char *key ) {
  return this->get_int( key, 0 );
}

int TtIniSection::get_int( const char *key, int default_int ) {
  return GetPrivateProfileInt( this->name, key, default_int, this->ini_file->get_file_name() );
}

int TtIniSection::get_str( const char *key, char *buf, int buf_size ) {
  return this->get_str( key, buf, buf_size, NULL );
}

int TtIniSection::get_str( const char *key, char *buf, int buf_size, const char *default_str ) {
  return GetPrivateProfileString( this->name, key, default_str, buf, buf_size, this->ini_file->get_file_name() );
}

bool TtIniSection::set_int( const char *key, int data ) {
  char buf[20];
  sprintf( buf, "%d", data );
  return this->set_str( key, buf );
}

bool TtIniSection::set_str( const char *key, const char *data ) {
  return WritePrivateProfileString( this->name, key, data, this->ini_file->get_file_name() );
}

void TtIniSection::flush() {
  this->ini_file->flush();
}
