#include "ttl_p.h"

// -- static -------------------------------------------------------------
int TtBasicSocket::static_error_id = 0;

bool TtBasicSocket::init() {
  WSADATA wsa;
  int status;

  status = WSAStartup( 2, &wsa );
  if ( status != 0 ) {
    TtBasicSocket::static_error_id = status;
    return false;
  }
  return true;
}

bool TtBasicSocket::cleanup() {
  if ( WSACleanup() == SOCKET_ERROR ) {
    TtBasicSocket::static_error_id = WSAGetLastError();
    return false;
  }
  return true;
}

int TtBasicSocket::get_static_error_id() {
  return TtBasicSocket::static_error_id;
}

// -- instance -----------------------------------------------------------
TtBasicSocket::TtBasicSocket() {
  this->sock = 0;
  this->timeout = 0;
  this->last_error_id = 0;
}

bool TtBasicSocket::set_timeout( int timeout ) {
  this->timeout = timeout;
  if ( setsockopt( this->sock,  SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(timeout) ) == SOCKET_ERROR ||
       setsockopt( this->sock,  SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout, sizeof(timeout) ) == SOCKET_ERROR ) {
    this->error_setting();
    return false;
  }
  return true;
}

int TtBasicSocket::get_timeout() {
  return this->timeout;
}


bool TtBasicSocket::connect( char *host, int port ) {
  int status;
  struct sockaddr_in sockAddrIn;
  struct hostent     *hostEnt;
  struct hostent     tempHostEnt;
  
  // {͔rׂA߂ǂ̂łȂ
  hostEnt = gethostbyname( host );

  if ( hostEnt == NULL ) {
    this->error_setting();
    return false;
  }
  tempHostEnt= *hostEnt;
  hostEnt = &tempHostEnt;

  sockAddrIn.sin_family = AF_INET;
  sockAddrIn.sin_addr = *((struct in_addr *)hostEnt->h_addr_list[0]);
  sockAddrIn.sin_port = htons( (unsigned short)port );
  
  status = ::connect( this->sock, (struct sockaddr*)&sockAddrIn, sizeof(sockAddrIn) );
  if ( status == SOCKET_ERROR ) {
    this->error_setting();
    return false;
  }
  return true;
}

bool TtBasicSocket::close() {
  if( shutdown( this->sock, SD_BOTH ) == SOCKET_ERROR ) {
    this->error_setting();
    return false;
  }
  if ( closesocket( this->sock ) == SOCKET_ERROR ) {
    this->error_setting();
    return false;
  }
  return true;
}


bool TtBasicSocket::read_int( int *data ) {
  if ( this->read_bytes( (char *)data, sizeof(int) ) ) {
    *(data) = (int)(ntohl( (unsigned long)(*(data)) ));
    return true;
  }
  return false;
}

bool TtBasicSocket::read_char( char *data ) {
  return this->read_bytes( data, sizeof(char) );
}

bool TtBasicSocket::read_bytes( char *data, int size ) {
  for ( int i = 0; i < size; ) {
    int recved = this->recv( &data[i], size - i, 0 );
    if ( recved <= 0 ) {
      this->error_setting();
      return false;
    }
    i += recved;
  }
  return true;
}

bool TtBasicSocket::read_line( char *data, int max ) {
  for ( int i = 0; i < max - 1; ++i ) {
    if ( !this->read_char( &data[i] ) ) { break; }
    if ( data[i] == '\n' ) {
      data[i + 1] = '\0';
      break;
    }
  }
  return true;
}

int TtBasicSocket::recv( char *buf, int len, int flags ) {
  return ::recv( this->sock, buf, len, flags );
}


bool TtBasicSocket::write_int( int data ) {
  unsigned long big_endian_data;
  big_endian_data = htonl( (unsigned long)data );
  return this->write_bytes( (char *)&big_endian_data, sizeof(int) );
}

bool TtBasicSocket::write_char( char data ) {
  return this->write_bytes( &data, sizeof(char) );
}

bool TtBasicSocket::write_bytes( const char *data, int size ) {
  for ( int i = 0; i < size; ) {
    int sent = this->send( &data[i], size - i, 0 );
    if ( sent <= 0 ) {
      this->error_setting();
      return false;
    }
    i += sent;
  }
  return true;
}

bool TtBasicSocket::write_string( const char *data ) {
  return this->write_bytes( data, strlen( data ) );
}

bool TtBasicSocket::write_line( const char *data ) {
  return this->write_string( data ) && this->write_char( '\n' );
}

int TtBasicSocket::send( const char *buf, int len, int flags ) {
  return ::send( this->sock, buf, len, flags );
}


int TtBasicSocket::get_error_id() {
  return this->last_error_id;
}

bool TtBasicSocket::get_error_message( char *buf, int max ) {
  return FormatMessage( FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM,
                 NULL,
                 this->last_error_id,
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                 buf,
                 max,
                 NULL );
}

void TtBasicSocket::error_setting() {
  this->last_error_id = WSAGetLastError();
}
