#include "ttl_p.h"

const int TtTreeItem::TABLE_SIZE = 13;

TtTreeItem::TtTreeItem() {
  this->tree = NULL;
  this->substance = NULL;
  this->parent = NULL;
  this->childs = NULL;
  this->next = NULL;
  this->child_image_index = -1;
  this->child_selected_image_index = -1;
}

TtTreeItem::TtTreeItem( TtTreeView *tree ) {
  this->tree = tree;
  this->substance = new TVINSERTSTRUCT;
  this->parent = NULL;
  this->childs = new TtTreeItem[TtTreeItem::TABLE_SIZE];
  this->next = NULL;
  this->child_image_index = -1;
  this->child_selected_image_index = -1;
}

TtTreeItem::~TtTreeItem() {
  if ( this->substance != NULL ) { delete this->substance; }
  if ( this->childs != NULL ) {
    for ( int i = 0; i < TtTreeItem::TABLE_SIZE; ++i ) {
      if ( this->childs[i].next != NULL ) { delete this->childs[i].next; }
    }
    delete [] this->childs;
  }
  if ( this->next != NULL ) { delete this->next; }
}

//-- collection systems --------------------------------------------------
bool TtTreeItem::add_child( TtTreeItem *child ) {
  if ( child->tree != this->tree ) { return false; }
  int hash = ((int)child->get_handle()) / 4 % TtTreeItem::TABLE_SIZE;
  TtTreeItem *current = &this->childs[hash];
  while ( current->next != NULL ) {
    current = current->next;
  }
  current->next = child;
  return true;
}

bool TtTreeItem::remove_child( TtTreeItem *child ) {
  if ( child->tree != this->tree ) { return false; }
  int hash = ((int)child->get_handle()) / 4 % TtTreeItem::TABLE_SIZE;
  TtTreeItem *current = &this->childs[hash];
  while ( current->next != NULL ) {
    if ( current->next == child ) {
      current->next = child->next;
      break;
    }
    current = current->next;
  }
  return true;
}

TtTreeItem *TtTreeItem::find_child( HTREEITEM item ) {
  int hash = ((int)item) / 4 % TtTreeItem::TABLE_SIZE;
  TtTreeItem *current = this->childs[hash].next;
  while ( current != NULL ) {
    if ( current->get_handle() == item ) {
      return current;
    }
    current = current->next;
  }
  return NULL;
}

TtTreeItem *TtTreeItem::find_descendant( HTREEITEM item ) {
  TtTreeItem *_tmp = this->find_child( item );
  if ( _tmp != NULL ) { return _tmp; }

  for ( int i = 0; i < TtTreeItem::TABLE_SIZE; ++i ) {
    TtTreeItem *current = this->childs[i].next;
    while ( current != NULL ) {
      _tmp = current->find_descendant( item );
      if ( _tmp != NULL ) { return _tmp; }
      current = current->next;
    }
  }
  return NULL;
}

//------------------------------------------------------------------------
bool TtTreeItem::do_get() {
  return TreeView_GetItem( this->tree->get_h_wnd(), &this->substance->item );
}

bool TtTreeItem::do_set() {
  return TreeView_SetItem( this->tree->get_h_wnd(), &this->substance->item );
}

HTREEITEM TtTreeItem::get_handle() {
  return this->substance->item.hItem;
}

TtTreeItem *TtTreeItem::get_parent() {
  return this->parent;
}

TtTreeItem *TtTreeItem::get_first_child() {
  return this->find_child( TreeView_GetChild( this->tree->get_h_wnd(), this->get_handle() ) );
}

TtTreeItem *TtTreeItem::get_next_brother() {
  return this->parent->find_child( TreeView_GetNextSibling( this->tree->get_h_wnd(), this->get_handle() ) );
}

TtTreeItem *TtTreeItem::get_prev_brother() {
  return this->parent->find_child( TreeView_GetPrevSibling( this->tree->get_h_wnd(), this->get_handle() ) );
}


void TtTreeItem::insert( TtTreeItem *item ) {
  item->substance->hParent = this->get_handle();
  item->substance->item.mask = 0;
  if ( this->child_image_index != -1 ) {
    item->substance->item.mask |= TVIF_IMAGE;
    item->substance->item.iImage = this->child_image_index;
  }
  if ( this->child_selected_image_index != -1 ) {
    item->substance->item.mask |= TVIF_SELECTEDIMAGE;
    item->substance->item.iSelectedImage = this->child_selected_image_index;
  }
  item->substance->item.hItem = TreeView_InsertItem( this->tree->get_h_wnd(), item->substance );
  this->add_child( item );
  item->parent = this;
}

void TtTreeItem::insert_first( TtTreeItem *item ) {
  item->substance->hInsertAfter = TVI_FIRST;
  this->insert( item );
}

void TtTreeItem::insert_last( TtTreeItem *item ) {
  item->substance->hInsertAfter = TVI_LAST;
  this->insert( item );
}

void TtTreeItem::insert_sorted( TtTreeItem *item ) {
  item->substance->hInsertAfter = TVI_SORT;
  this->insert( item );
}

TtTreeItem *TtTreeItem::insert_text( char *str ) {
  TtTreeItem *item = new TtTreeItem( this->tree );
  this->insert( item );
  item->set_text( str );
  return item;
}

void TtTreeItem::sort_childlen() {
  TreeView_SortChildren( this->tree->get_h_wnd(), this->get_handle(), FALSE );
}

void TtTreeItem::sort_childlen_recursive() {
  TreeView_SortChildren( this->tree->get_h_wnd(), this->get_handle(), TRUE );
}


bool TtTreeItem::unlink( TtTreeItem *item ) {
  this->remove_child( item );
  return TreeView_DeleteItem( this->tree->get_h_wnd(), item->get_handle() );
}

bool TtTreeItem::remove( TtTreeItem *item ) {
  bool _tmp = this->unlink( item );
  delete item;
  return _tmp;
}

void TtTreeItem::remove_all() {
  for ( int i = 0; i < TtTreeItem::TABLE_SIZE; ++i ) {
    TtTreeItem *root = &this->childs[i];
    while( root->next != NULL ) {
      this->remove( root->next );
    }
    root->next = NULL;
  }
}


void TtTreeItem::set_text( char *str ) {
  this->substance->item.mask = TVIF_TEXT;
  this->substance->item.pszText = str;
  this->do_set();
}


void TtTreeItem::get_text( char *buf, int buf_size ) {
  this->substance->item.mask = TVIF_TEXT;
  this->substance->item.pszText = buf;
  this->substance->item.cchTextMax = buf_size;
  this->do_get();
}

void TtTreeItem::set_edit_mode() {
  TreeView_EditLabel( this->tree->get_h_wnd(), this->get_handle() );
}

void TtTreeItem::set_select() {
  TreeView_SelectItem( this->tree->get_h_wnd(), this->get_handle() );
}


void TtTreeItem::expand() {
  TreeView_Expand( this->tree->get_h_wnd(), this->get_handle(), TVE_EXPAND );
}

void TtTreeItem::collapse() {
  TreeView_Expand( this->tree->get_h_wnd(), this->get_handle(), TVE_COLLAPSE );
}

void TtTreeItem::toggle_expand() {
  TreeView_Expand( this->tree->get_h_wnd(), this->get_handle(), TVE_TOGGLE );
}


void TtTreeItem::set_image_index( int index ) {
  this->substance->item.mask = TVIF_IMAGE;
  this->substance->item.iImage = index;
  this->do_set();
}

void TtTreeItem::set_selected_image_index( int index ) {
  this->substance->item.mask = TVIF_SELECTEDIMAGE;
  this->substance->item.iSelectedImage = index;
  this->do_set();
}

void TtTreeItem::set_both_image_index( int index ) {
  this->substance->item.mask = TVIF_IMAGE | TVIF_SELECTEDIMAGE;
  this->substance->item.iImage = index;
  this->substance->item.iSelectedImage = index;
  this->do_set();
}

int TtTreeItem::get_child_image_index() {
  return child_image_index;
}
void TtTreeItem::set_child_image_index( int index ) {
  this->child_image_index = index;
}

int TtTreeItem::get_child_selected_image_index() {
  return child_selected_image_index;
}
void TtTreeItem::set_child_selected_image_index( int index ) {
  this->child_selected_image_index = index;
}
